PROGRAM u_profile

  ! - Reads monthly mean profiles of zonal wind from three equatorial
  !   stations:
  !      Canton Island, 02 46 S / 171 43 W, Jan.1953-Aug.1967
  !      Gan/Maledives, 00 41 S /  73 09 E, Sep.1967-Dec.1975
  !      Singapore,     01 22 N / 103 55 E, Jan.1976 onwards
  !
  ! - Interpolates data of 1953 to 1986, given at 7 levels from
  !   70 to 10 hPa, to higher resolution grid with 14 levels from 90 to
  !   10 hPa, as used 1987 onwards.
  !
  ! - Extends the QBO from 15 hPa to 10 hPa, where necessary. The upward
  !   extension is done by an upward propagation of the time series at
  !   15 hPa assuming a propagation rate of 2km/month.
  !
  ! - Extends the QBO from 10 hPa, the uppermost level of observations,
  !   to the upper stratosphere. The upward extension is done by an upward
  !   propagation of the time series at 10 hPa assuming a propagation
  !   rate of 2km/month and an exponential decay of amplitude using a
  !   length scale of 10km.

  !=============================================================================

  USE mo_kind,        ONLY: rs
  USE mo_interpolate, ONLY: interpolate_index,&
       &                    interpolate_polynom,&
       &                    interpolate_reset

  IMPLICIT NONE

  ! Grids
  ! -----
  !   pressure levels grids in hPa
  !   low  resolution:
  INTEGER,  PARAMETER   :: nlowres=7
  REAL(rs), PARAMETER   :: plowres(nlowres)=&
       & (/          10,   15,20,   30,   40,   50,   70      /)
  !
  !   high resolution:
  INTEGER,  PARAMETER   :: nhighres=14
  REAL(rs), PARAMETER   :: phighres(nhighres)=&
       & (/          10,12,15,20,25,30,35,40,45,50,60,70,80,90/)
  !
  !   extended high resolution:
  INTEGER,  PARAMETER   :: nextres=19
  REAL(rs), PARAMETER   :: pextres(nextres)=&
       & (/3,4,5,6,8,10,12,15,20,25,30,35,40,45,50,60,70,80,90/)
  INTEGER,    PARAMETER :: iextlev10=6  ! index of 10 hPa level
  INTEGER,    PARAMETER :: iextlev15=8  ! index of 15 hPa level
  INTEGER,    PARAMETER :: iextlev70=17 ! index of 70 hPa level

  ! Observations of zonal wind
  ! --------------------------
  ! January 1953 to August 1967: Canton Island
  ! 15-70 hPa in Jan.53-Dec.55
  ! 10-70 hPa in Jan.56-Aug.67
  CHARACTER*40          :: file1='Canton_u_195301_196708'
  INTEGER, PARAMETER    :: nmonth1a=(1955-1952)*12
  INTEGER, PARAMETER    :: nmonth1b=(1966-1955)*12+8
  INTEGER, PARAMETER    :: nmonth1=nmonth1a+nmonth1b
  REAL(rs)              :: u1low(nlowres,nmonth1)

  ! September 1967 to December 1975: Gan/Maledives
  CHARACTER*40          :: file2='Gan_u_196709_197512'
  INTEGER, PARAMETER    :: nmonth2=4+(1975-1967)*12
  REAL(rs)              :: u2low(nlowres,nmonth2)

  ! January 1976 to December 1986: Singapore
  CHARACTER*40          :: file3='Singapore_u_197601_198612'
  INTEGER, PARAMETER    :: nmonth3=(1986-1975)*12
  REAL(rs)              :: u3low(nlowres,nmonth3)

  ! January 1987 to November 2012: Singapore
  CHARACTER*40          :: file4='Singapore_u_198701_201211'
  INTEGER, PARAMETER    :: nmonth4=(2011-1986)*12+11
  REAL(rs)              :: u4high(nhighres,nmonth4)
  REAL(rs)              :: u4low(nlowres,nmonth4)

  ! time grid for full time
  ! -----------------------
  INTEGER, PARAMETER    :: nmonth=nmonth1+nmonth2+nmonth3+nmonth4
  REAL(rs)              :: tgrid(nmonth)


  ! composed, interpolated and extended wind profiles
  ! -------------------------------------------------
  REAL(rs)              :: ulow(nlowres,nmonth)
  REAL(rs)              :: uhigh(nhighres,nmonth)
  REAL(rs)              :: uext(nextres,nmonth)

  ! value for missing wind observation
  ! ----------------------------------
  REAL(rs), PARAMETER   :: undef=99.9


  ! Vertical extension above 10 hPa
  ! -------------------------------
  INTEGER               :: firstmonth
  INTEGER               :: lastmonth
  REAL(rs), ALLOCATABLE :: tgrid1(:) ! unshifted time grid
  REAL(rs), ALLOCATABLE :: tgrid2(:) ! shifted time grid

  REAL(rs),PARAMETER    :: h=7.0     ! scale height of pressure (km)
  INTEGER               :: iref      ! index of reference p level in pextres(:)
  REAL(rs)              :: zreflev   ! height of reference pressure level (km)
  REAL(rs)              :: zextlev   ! height of pressure level (km)
  REAL(rs)              :: dz        ! = zextlev-zreflev (km)

  REAL(rs)              :: hqbo      ! scale height of QBO decay (km)
  REAL(rs)              :: w         ! QBO propagation rate (km/month)
  REAL(rs)              :: dt        ! = dz/w (month)


  ! Auxuliary variables
  ! -------------------
  INTEGER :: i,j,i2first,i2last     ! Indices
  INTEGER :: idate                  ! date in YYYYMM format
  REAL    :: d                      ! REAL variable of default kind


  !=============================================================================

  ! Read data from files 
  ! ---------------------
  ! January 1953 to August 1967: u profiles at Canton Island
  CALL read_ulowres(TRIM(ADJUSTL(file1)),nlowres,nmonth1,u1low)
  !
  ! September 1967 to December 1975: u profiles at Gan/Maledives
  CALL read_ulowres(TRIM(ADJUSTL(file2)),nlowres,nmonth2,u2low)
  !
  ! January 1976 to December 1986: u profiles at Singapore
  CALL read_ulowres(TRIM(ADJUSTL(file3)),nlowres,nmonth3,u3low)
  !
  ! January 1987 onwards: u profiles at Singapore at high resolution
  CALL read_uhighres(TRIM(ADJUSTL(file4)),nhighres,nmonth4,u4high)

  ! copy high res. profile from Singapore to low res. profile
  u4low(1,:)=u4high(1,:)  ! 10 hPa
  u4low(2,:)=u4high(3,:)  ! 15 hPa
  u4low(3,:)=u4high(4,:)  ! 20 hPa
  u4low(4,:)=u4high(6,:)  ! 30 hPa
  u4low(5,:)=u4high(8,:)  ! 40 hPa
  u4low(6,:)=u4high(10,:) ! 50 hPa
  u4low(7,:)=u4high(12,:) ! 70 hPa

  !=============================================================================

  ! compile full low resolution time series
  ! ---------------------------------------
  i=  1;  j=  nmonth1;  ulow(:,i:j)=u1low(:,:)
  i=j+1;  j=j+nmonth2;  ulow(:,i:j)=u2low(:,:)
  i=j+1;  j=j+nmonth3;  ulow(:,i:j)=u3low(:,:)
  i=j+1;  j=j+nmonth4;  ulow(:,i:j)=u4low(:,:)

  ! write low resolution profiles to GrADS data file
  OPEN(unit=21,file='u_profile_lowres_195301-201211',form='unformatted')
  DO i=1,nmonth
     DO j=1,nlowres
        d=ulow(j,i) ! copy to default precision REAL variable
        WRITE(21) d
     END DO
  END DO
  CLOSE(21)

  ! write low resolution profiles to text data file
  OPEN(unit=21,file='u_profile_lowres_195301-201211.txt',form='formatted')
  WRITE(21,*) 'Equatorial zonal wind on ',nlowres,' pressure levels [m/s]'
  WRITE(21,*) 'see original files for  months with less than 10 ascents'
  WRITE(21,*) '99.9=undefined'
  WRITE(21,*) '> Canton Island, 02 46 S / 171 43 W, Jan.1953-Aug.1967'
  WRITE(21,*) '> Gan/Maledives, 00 41 S /  73 09 E, Sep.1967-Dec.1975'
  WRITE(21,*) '> Singapore,     01 22 N / 103 55 E, Jan.1976 onwards'
  WRITE(21,*) 'Input files compiled by B. Naujokat, M. Kunze, FUB, Berlin'
  WRITE(21,*) '[Labitzke et al., 2002]'
  WRITE(21,*) 'Postprocessing: M. A. Giorgetta, MPI-M, Hamburg'
  WRITE(21,*) '--------------------------------------------------------'
  WRITE(21,'(a6,7f7.2)')  'p[hPa]',plowres(:)
  WRITE(21,'(a6)')        'YYYYMM'
  DO i=1,nmonth
     idate=(1953+(i-1)/12)*100+(i-((i-1)/12)*12)
     WRITE(21,'(i6,7f7.2)') idate,(ulow(j,i),j=1,7)
  END DO
  CLOSE(21)

  !=============================================================================

  ! interpolate low resolution time series to high resolution grid
  ! --------------------------------------------------------------
  ! January 1953 to December 1955: from 15 to 70 hPa
  CALL interpolate_index(plowres(2:),phighres,i2first,i2last)
  DO i=1,nmonth1a
     uhigh(:,i)=interpolate_polynom(ulow(2:,i),3,undef)
  END DO
  CALL interpolate_reset
  !
  ! January 1956 to December 1986: from 10 to 70 hPa
  CALL interpolate_index(plowres,phighres,i2first,i2last)
  DO i=nmonth1a+1,nmonth1+nmonth2+nmonth3
     uhigh(:,i)=interpolate_polynom(ulow(:,i),3,undef)
  END DO
  CALL interpolate_reset
  !
  ! January 1987 to December 2004: copy data
  i= nmonth1+nmonth2+nmonth3+1
  j= nmonth1+nmonth2+nmonth3+nmonth4
  uhigh(:,i:j)=u4high(:,:)

  ! write high resolution profiles to GrADS data file
  OPEN(unit=21,file='u_profile_highres_195301-201211',form='unformatted')
  DO i=1,nmonth
     DO j=1,nhighres
        d=uhigh(j,i) ! copy to default precision REAL variable
        WRITE(21) d
     END DO
  END DO
  CLOSE(21)

  ! write high resolution profiles to text data file
  OPEN(unit=21,file='u_profile_highres_195301-201211.txt',form='formatted')
  WRITE(21,*) 'Equatorial zonal wind on ',nlowres,' pressure levels [m/s]'
  WRITE(21,*) 'see original files for  months with less than 10 ascents'
  WRITE(21,*) '99.9=undefined'
  WRITE(21,*) '> Canton Island, 02 46 S / 171 43 W, Jan.1953-Aug.1967'
  WRITE(21,*) '> Gan/Maledives, 00 41 S /  73 09 E, Sep.1967-Dec.1975'
  WRITE(21,*) '> Singapore,     01 22 N / 103 55 E, Jan.1976 onwards'
  WRITE(21,*) 'Input files compiled by B. Naujokat, M. Kunze, FUB, Berlin'
  WRITE(21,*) '[Labitzke et al., 2002]'
  WRITE(21,*) 'Postprocessing: M. A. Giorgetta, MPI-M, Hamburg'
  WRITE(21,*) '--------------------------------------------------------'
  WRITE(21,'(a6,14f7.2)') 'p[hPa]',phighres(:)
  WRITE(21,'(a6)')        'YYYYMM'
  DO i=1,nmonth
     idate=(1953+(i-1)/12)*100+(i-((i-1)/12)*12)
     WRITE(21,'(i6,14f7.2)') idate,(uhigh(j,i),j=1,14)
  END DO
  CLOSE(21)

  !=============================================================================

  ! construct vertically extended u profiles by propagation operation
  ! -----------------------------------------------------------------
  ! reset uext at p< 10hPa
  uext(1:iextlev10-1,:)=undef

  ! copy uhigh at p>=10hPa
  uext(iextlev10:iextlev10-1+nhighres,:)=uhigh(:,:)

  ! time grid
  tgrid(:)=(/(i,i=1,nmonth)/)

  ! propagate QBO from the selected reference level to higher levels
  ! propagation occurs with constant rate w = 2km/month
  ! additionally the amplitude at levels above 10 hPa is damped from one
  ! level to the next by a factor of 0.8326, such that the QBO amplitude
  ! at the uppermost level of the extended grid is reduced

  w   = 2.0 ! QBO propagation rate (km/month)
  hqbo=10.0 ! scale height of upward QBO decay above 10hPa (km)

  ! propagate from 15 hPa up to 10 hPa for Jan.1953 - Dec.1955
  ! ----------------------------------------------------------
  firstmonth=1                          ! index of January 1953
  lastmonth=nmonth1a                    ! index of December 1955
  ALLOCATE(tgrid1(lastmonth-firstmonth+1))
  ALLOCATE(tgrid2(lastmonth-firstmonth+1))
  tgrid1=(/(i,i=firstmonth,lastmonth)/) ! unshifted time grid

  iref=iextlev15                        ! index of 15 hPa level
  zreflev=h*LOG(1000./pextres(iref))    ! height of reference level

  DO i=iextlev10,iref-1
     zextlev=h*LOG(1000./pextres(i))
     dz=zextlev-zreflev
     dt=dz/w
     tgrid2(:)=tgrid1(:)+dt
     CALL interpolate_index(tgrid,tgrid2,i2first,i2last)
     uext(i,firstmonth:lastmonth)=&
          & interpolate_polynom(uext(iref,:),1,undef)
     CALL interpolate_reset
  END DO

  DEALLOCATE(tgrid1,tgrid2)

  ! propagate from 10 hPa to uppermost level for all months
  ! -------------------------------------------------------
  firstmonth=1                          ! index of January 1953
  lastmonth=nmonth                      ! index of last month
  ALLOCATE(tgrid1(lastmonth-firstmonth+1))
  ALLOCATE(tgrid2(lastmonth-firstmonth+1))
  tgrid1=(/(i,i=firstmonth,lastmonth)/) ! time grid in months

  iref=iextlev10                        ! index of 10 hPa level
  zreflev=h*LOG(1000./pextres(iref))    ! hight of reference level

  DO i=1,iref-1
     zextlev=h*LOG(1000./pextres(i))
     dz=zextlev-zreflev
     dt=dz/w
     tgrid2(:)=tgrid1(:)+dt
     CALL interpolate_index(tgrid,tgrid2,i2first,i2last)
     uext(i,firstmonth:lastmonth)=&
          & interpolate_polynom(uext(iref,:),1,undef)
     WHERE (uext(i,firstmonth:lastmonth)/=undef) &
          & uext(i,firstmonth:lastmonth)=        &
          & uext(i,firstmonth:lastmonth)*EXP(-dz/hqbo)
     CALL interpolate_reset
  END DO

  DEALLOCATE(tgrid1,tgrid2)

  ! smooth at 10 hPa by a 1-2-1 filter, once at level iref, twice at level iref-1, ...
  ! ----------------------------------
  DO i=iref,1,-1
     j=iref-i+1
     WHERE (uext(1:i,firstmonth+j-1:lastmonth-j-1)/=undef .AND. &
          & uext(1:i,firstmonth+j:lastmonth-j)/=undef     .AND. &
          & uext(1:i,firstmonth+j+1:lastmonth-j+1)/=undef) &
          &     uext(1:i,firstmonth+j:lastmonth-j)=                               &
          &                        (      uext(1:i,firstmonth+j-1:lastmonth-j-1)+ &
          &                         2._rs*uext(1:i,firstmonth+j:lastmonth-j)+     &
          &                               uext(1:i,firstmonth+j+1:lastmonth-j+1)  )/4._rs
  END DO

  !=============================================================================

  ! propagate QBO to lower levels from January 1953 to December 1987
  ! propagation occurs with constant rate w = 1km/month
  ! additionally the amplitude is damped from one level to the next
  ! by a factor of 0.8326, such that the amplitude at the lowermost
  ! level of the extended grid is reduced 

  w   = 1.0 ! propagation rate (km/month)
  hqbo=-2.0 ! scale height of upward QBO decay below 70hPa (km)

  ! propagate from 70 hPa down to 90 hPa for Jan.1953 - Dec.1986
  ! ------------------------------------------------------------
  firstmonth=1
  lastmonth=nmonth1+nmonth2+nmonth3
  ALLOCATE(tgrid1(lastmonth-firstmonth+1))
  ALLOCATE(tgrid2(lastmonth-firstmonth+1))
  tgrid1=(/(i,i=firstmonth,lastmonth)/) ! time grid in months

  iref=iextlev70                        ! index of 70 hPa level
  zreflev=h*LOG(1000./pextres(iref))    ! hight of reference level

  DO i=iextlev70+1,nextres
     zextlev=h*LOG(1000./pextres(i))
     dz=zextlev-zreflev
     dt=dz/w
     tgrid2(:)=tgrid1(:)+dt
     CALL interpolate_index(tgrid,tgrid2,i2first,i2last)
     uext(i,firstmonth:lastmonth)=&
          & interpolate_polynom(uext(iref,:),1,undef)
     WHERE (uext(i,firstmonth:lastmonth)/=undef) &
          & uext(i,firstmonth:lastmonth)=        &
          & uext(i,firstmonth:lastmonth)*EXP(-dz/hqbo)
     CALL interpolate_reset
  END DO

  DEALLOCATE(tgrid1,tgrid2)

  ! write extended high resolution profiles to GrADS data file
  OPEN(unit=21,file='u_profile_extres_195301-201211',form='unformatted')
  DO i=1,nmonth
     DO j=1,nextres
        d=uext(j,i) ! copy to default precision REAL variable
        WRITE(21) d
     END DO
  END DO
  CLOSE(21)

  OPEN(unit=21,file='u_profile_extres_195301-201211.txt',form='formatted')
  WRITE(21,*) 'Equatorial zonal wind on ',nextres,' pressure levels [m/s]'
  WRITE(21,*) 'see original files for  months with less than 10 ascents'
  WRITE(21,*) '99.9=undefined'
  WRITE(21,*) '> Canton Island, 02 46 S / 171 43 W, Jan.1953-Aug.1967'
  WRITE(21,*) '> Gan/Maledives, 00 41 S /  73 09 E, Sep.1967-Dec.1975'
  WRITE(21,*) '> Singapore,     01 22 N / 103 55 E, Jan.1976 onwards'
  WRITE(21,*) 'Input files compiled by B. Naujokat, M. Kunze, FUB, Berlin'
  WRITE(21,*) '[Labitzke et al., 2002]'
  WRITE(21,*) 'Postprocessing: M. A. Giorgetta, MPI-M, Hamburg'
  WRITE(21,*) '--------------------------------------------------------'
  WRITE(21,'(a6,19f7.2)') 'p[hPa]',pextres(:)
  WRITE(21,'(a6)')        'YYYYMM'
  DO i=1,nmonth
     idate=(1953+(i-1)/12)*100+(i-((i-1)/12)*12)
     WRITE(21,'(i6,19f7.2)') idate,(uext(j,i),j=1,19)
  END DO
  CLOSE(21)

CONTAINS

  !=============================================================================

  SUBROUTINE read_ulowres(ifile,nlev,nmonth,u)

    ! returns u profiles
    !
    ! on file  : u in 0.1m/s, ordered from 70 hPa to 10 hPa
    ! in output: u in m/s,    ordered from 10 hPa to 70 hPa

    CHARACTER(*),INTENT(in) :: ifile
    INTEGER,     INTENT(in) :: nlev,nmonth
    REAL(rs),  INTENT(out):: u(nlev,nmonth)

    INTEGER :: i,iu(nlev),j

    OPEN (unit=11,file=ifile,form='formatted',status='old')

    ! Skip 8 header lines

    DO i=1,8
       READ(11,*)
    END DO

    DO i=1,nmonth
       READ(11,'(4x,1x,7(i5,2x))') iu(:)
       DO j=1,nlev
          u(nlev+1-j,i)=float(iu(j))/10.
       END DO
    END DO

    CLOSE(11)

  END SUBROUTINE read_ulowres

  !=============================================================================

  SUBROUTINE read_uhighres(ifile,nlev,nmonth,u)

    ! returns u profiles
    !
    ! on file  : u in 0.1m/s, ordered from 10 hPa to 100 hPa
    ! in output: u in m/s,    ordered from 10 hPa to  90 hPa

    CHARACTER(*),INTENT(in) :: ifile
    INTEGER,     INTENT(in) :: nlev,nmonth
    REAL(rs),  INTENT(out):: u(nlev,nmonth)

    INTEGER :: i,iu(12),j,k

    OPEN (unit=11,file=ifile,form='formatted',status='old')

    ! Skip 3 header lines
    READ(11,*)
    READ(11,*)
    READ(11,*)

    ! READ years 1987 to 1996
    DO i=1987,1996
       j=(i-1987)*12
       READ(11,*)
       READ(11,*)
       READ(11,*)
       DO k=1,nlev ! get 10-90 hPa wind data
          READ(11,'(2x,1x,12i5)') iu(:)
          u(k,j+1:j+12)=float(iu(:))/10.
       ENDDO
    ENDDO

    ! READ years 1997 to 2006
    DO i=1997,2011
       j=(i-1987)*12
       READ(11,*)
       READ(11,*)
       READ(11,*)
       DO k=1,nlev ! get 10-90 hPa wind data
          READ(11,'(2x,1x,12i5)') iu(:)
          u(k,j+1:j+12)=float(iu(:))/10.
       ENDDO
       READ(11,*)  ! skip 100 hPa wind data
    ENDDO

    ! READ year 2012 January to November (months 1 to 11)
    i=2012
    j=(i-1987)*12
    READ(11,*)
    READ(11,*)
    READ(11,*)
    DO k=1,nlev ! get 10-90 hPa wind data
       READ(11,'(2x,1x,12i5)') iu(1:11)   ! (months 1 to 11)
       u(k,j+1:j+11)=float(iu(1:11))/10.  ! (months 1 to 11)
    ENDDO
    READ(11,*)  ! skip 100 hPa wind data

    CLOSE(11)

  END SUBROUTINE read_uhighres


END PROGRAM u_profile
